Package: doRNG
Type: Package
Title: Generic Reproducible Parallel Backend for 'foreach' Loops
Version: 1.8.6.3
Date: 2026-02-05
Encoding: UTF-8
Authors@R: c(
    person("Renaud", "Gaujoux",
    email = "renozao@protonmail.com", role = c("aut")),
    person(given = c("Emilio", "L."), family = "Sáenz Guillén",
       email = "Emilio.Saenz-Guillen@citystgeorges.ac.uk", role = "cre")
    )
Description: Provides functions to perform
    reproducible parallel foreach loops, using independent
    random streams as generated by L'Ecuyer's combined
    multiple-recursive generator [L'Ecuyer (1999), <DOI:10.1287/opre.47.1.159>].
    It enables to easily convert standard '%dopar%' loops into
    fully reproducible loops, independently of the number
    of workers, the task scheduling strategy, or the chosen
    parallel environment and associated foreach backend.
License: GPL (>= 2)
LazyLoad: yes
URL: https://github.com/emilioluissaenzguillen/doRNG
BugReports: https://github.com/emilioluissaenzguillen/doRNG/issues
Depends: R (>= 3.0.0), foreach, rngtools (>= 1.5)
Imports: stats, utils, iterators
Suggests: doParallel, doMPI, doRedis, rbenchmark, knitr, rbibutils (>=
        1.3), testthat, covr
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-02-05 13:57:41 UTC; emili
Author: Renaud Gaujoux [aut],
  Emilio L. Sáenz Guillén [cre]
Maintainer: Emilio L. Sáenz Guillén <Emilio.Saenz-Guillen@citystgeorges.ac.uk>
Repository: CRAN
Date/Publication: 2026-02-05 15:50:03 UTC
