# Generated by roxygen2: do not edit by hand

export("%dorng%")
export(doRNGversion)
export(registerDoRNG)
import(foreach)
import(rngtools)
import(stats)
importFrom(iterators,iter)
importFrom(utils,compareVersion)
importFrom(utils,head)
importFrom(utils,packageVersion)
